# Description
# Ra-Based SGD Modeling Box Model with 2 Tracers & 1 Source

# This script includes data from the following paper: Luo, X.; Jiao, J.J.; Moore, W.S.; Ming Lee, C. Submarine groundwater discharge estimation in an urbanized embayment in Hong Kong via short-lived radium isotopes and its implication of nutrient loadings and primary production. Mar. Pollut. Bull. 2014, 82, 144–154.

# Instructions

# This script calculates the SGD flux and residence time to a coastal location using 
# a box model and equations described in Lecher et al, 2015 & Lecher et al, 2016.  Full references 
# are provided at the bottom of this script.  Please cite those articles when using this model. This 
# model uses two radium istopes (Ra1 and Ra2).  

# This script uses the Lawson-Hanson algorithm for non-negative least squares to solve for the contribution
# of sources and the nnls packge must be downloaded first by typing "install.packages("nnls")" into
# the console first.  You only need to install the package once.

# Email Alanna Lecher at alecher@lynn.edu with questions.

# Input the values of the variables to the right of the arrows below:

# Half-lives of the Ra isotopes in days:

HL_Ra1 <- 3.6
HL_Ra2 <- 11.4

# Volume of box in m^3

V <-156000000

# Ra activities in dpm/100L in the source (e.g. groundwater)

Ra1_S <- 4385
Ra2_S <- 109

# Ra activities in dpm/100L in the ocean box

Ra1_B <- 69.85
Ra2_B <- 46.19
  
# Ra offshore activities in dpm/100L

Ra1_O <- 69.85-42.17
Ra2_O <- 46.19-0.38


# DO NOT EDIT BELOW THIS LINE UNELSS YOU ARE ADDING UNKNOWNS OR TRACERS

# Call the non-negative least squares package
library(nnls)

# Calculate the decays constants of the isotopes

DC_Ra1 = log(2)/HL_Ra1
DC_Ra2 = log(2)/HL_Ra2

# Convert Ra activities to dmp/m^3 for consistent units

Ra1_s = Ra1_S*10
Ra2_s = Ra2_S*10
Ra1_b = Ra1_B*10
Ra2_b = Ra2_B*10
Ra1_o = Ra2_O*10
Ra2_o = Ra2_O*10

# Fill matrix of concentrations for GW and Advection for the equation Groundwater - Advection = Radioactive Decay
A = matrix(data = c(Ra1_s, -(Ra1_b-Ra1_o)*V, Ra2_s, -(Ra2_b-Ra2_o)*V), nrow = 2, ncol = 2, byrow = TRUE)

#Add in other radium sources, river+riverSPM+SPM
Raex1 <- (7.8*10^7+3.5*10^6+1.4*10^8)
Raex2 <- (1.7*10^6+1.8*10^4+2.7*10^5)

# Fill the vector of radiaoctive decay
b = c(Ra1_b*V*DC_Ra1-Raex1, Ra2_b*V*DC_Ra2-Raex2)

# Solve for flux and residence time using the Lawson Hanson non-negative least squares method
x = nnls(A, b)

# Prints the solutions

t = x[[1]][1]
u = x[[1]][2]
u2 = 1/u

print(paste('SGD flux in m^3/day =', t))
print(paste('residence time in days =', u2))

print(paste('residual sum of squares =', x[2]))



#References

#Lecher, A., Fisher., A., Paytan., A., (2016) Submarine groundwater discharge in Northern Monterey 
#Bay,California: Evaluation by mixing and mass balance models, (2016), Marine Chemistry.

#Lecher, A. Kessler, J., Sparrow, K., Garcia-Tigeros, F., Dimova, N., Murray, J., Tulaczyk, S., 
#Paytan, A., (2015) Methane Transport through submarine groundwater discharge to the North Pacific #and Arctic Ocean at two Alaskan sites, Limnology and Oceanography, Methane Special Issue.


#Licence

#Copyright 2022 Alanna L Lecher, PhD

#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
#associated documentation files (the "Software"), to deal in the Software without restriction, 
#including without limitation the rights to use, copy, modify, merge, publish, distribute, 
#sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
#furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all copies or substantial 
#portions of the Software."AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
#NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
#NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES 
#OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
#CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# End of script