# Description
# Ra-Based Mixing Model with 2 Tracers & 3 Sources

# This script includes data from the following paper: Su, N.; Burnett, W.C.; MacIntyre, H.L.; Liefer, J.D.; Peterson, R.N.; Viso, R. Natural Radon and Radium Isotopes for Assessing Groundwater Discharge into Little Lagoon, AL: Implications for Harmful Algal Blooms. Estuaries and Coasts 2014, 37, 893–910.

# Instructions

# This script calculates the relative contributions (as fractions of 1) of different water sources to a 
# coastal location using a mixing model and equations described in Lecher in review for Hydrology and Lecher et al, 2016.  Full references 
# are provided at the bottom of this script.  Please cite that article when using this model. It uses two 
# tracers and assumes there are only two major sources of that tracer.
# The concentration of tracer 1 (T1), tracer 2 (T2), and tracer 3 (T3) in source water 1 (S1), source water 2 (S2), and
# source water 3 (S3) and at the Locataion (L) are known.  The concentrations should be well scaled, meaning
# the values are of similar order of magnitude. 

# This script uses the Lawson-Hanson algorithm for non-negative least squares to solve for the contribution
# of sources and the nnls packge must be downloaded first by typing "install.packages("nnls")" into
# the console first.  You only need to install the package once. 

# Fractions that do not add up close to 1 or with large residuals inidcate this model is a poor fit to the data.  
# The model will iteratively output the results of each location in the console and a summary bar chart

# Email Alanna Lecher at alecher@lynn.edu with questions.

# Input the values of the variables to the right of the arrows below:

#Ra226
# Tracer 1 concentration in Source 1 (shallow groundwater)
S1_T1 <- 49.1
# Tracer 1 concentration in Source 2 (deep groundwater)
S2_T1 <- 51.95
# Tracer 1 concentration in Source 3 (seawater)
S3_T1 <- 8.0
# A vector of Tracer 1 concentrations at each Location, with values separated by commas in the parantheses, must be the same length as L_T2 and L_T3
L_T1 <- c(13.0, 14.1, 17.4, 14.7, 30.9)

#Ra228
# Tracer 2 concentration in Source 1
S1_T2 <- 52.4
# Tracer 2 concentration in Source 2
S2_T2 <- 317
# Tracer 2 concentration in Source 3
S3_T2 <- 5.6
# A vector of Tracer 2 concentrations at each Location, with values separated by commas in the parantheses
L_T2 <- c(25.2, 37.5, 45.1, 51.1, 116.0)


# DO NOT EDIT BELOW HERE

# Call the non-negative least squares package
library(nnls)

# Builds the matrix of known tracer concentration in the sources
A = matrix(data = c(S1_T1, S2_T1, S3_T1, 
                    S1_T2, S2_T2, S3_T2, 
                    1, 1, 1), nrow = 3, ncol = 3, byrow = TRUE)

#Build output vectors
S1_Output <- numeric()
S2_Output <- numeric()
S3_Output <- numeric()

# Builds the vector of tracer cocnentrations at the location
l = length(L_T1)
Ticker = 1

# Starts loop to fill location vectors & solve for each location
while (Ticker <= l) {
  
# Print the iteration of the loop for lables in the output
print(Ticker)

# Assign new values for next iteration    
Tracer1 = L_T1[Ticker]
Tracer2 = L_T2[Ticker]
  
b = c(Tracer1, Tracer2, 1)
  
# Uses the least squares method to solve for the unkwnonw source water contributions
x = nnls(A, b)
  
# Prints the contributions of the sources
  
t = x[[1]][1]
u = x[[1]][2]
w = x[[1]][3]
  
print(paste('source 1 fraction =', t))
print(paste('source 2 fraction =', u))
print(paste('source 3 fraction =', w))
  
print(paste('residual sum of squares =', x[2]))
  
# Update the output vectors
R1 <- c(S1_Output, t)
R2 <- c(S2_Output, u)
R3 <- c(S3_Output, w)
  
S1_Output <- R1
S2_Output <- R2
S3_Output <- R3
  
# Move the loop forward 1 iteration
Ticker = Ticker + 1
}

# Build a matrix of output for plotting

Results <- matrix(data = c(S1_Output, S2_Output, S3_Output), nrow = 3, ncol = l, byrow = TRUE)

# Plot the results
barplot(Results, main = "Stacked barchart", ylab = "Fraction", xlab = "Location")


# References

#Lecher, A. Open-source Code for Radium-Derived Ocean Modeling:Project Open RaDOM, in review for Hydrology

#Lecher, A., Fisher., A., Paytan., A., (2016) Submarine groundwater discharge in Northern Monterey 
#Bay,California: Evaluation by mixing and mass balance models, (2016), Marine Chemistry.

#Licence

#Copyright 2022 Alanna L Lecher, PhD

#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
#associated documentation files (the "Software"), to deal in the Software without restriction, 
#including without limitation the rights to use, copy, modify, merge, publish, distribute, 
#sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
#furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all copies or substantial 
#portions of the Software."AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
#NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
#NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES 
#OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
#CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# End of script
#

